#-*- coding:utf-8 -*-

import os
import os.path as Path
import operator

from base import EnhacModule

PATH_DELIM = ":"

'''
Include order:
    - file directory
    - //:add_inc_path ../inc/   <-- relative to source dir
    - ../.enhac_conf  (general project config)
    - ENHAC_PATH
'''


class Includer(EnhacModule):
    def __init__(self, enhac):
        self.enhac = enhac
        self.includes = []

        self.source_dir = Path.dirname(Path.abspath(self.enhac.fname))
        self.dirs_from_directives = []

        self.dirs_from_project_config = []
        if self.enhac.config.general.include_paths:
            conf_base = Path.dirname(Path.abspath(self.enhac.config.filename))
            self.dirs_from_project_config = (self.enhac.config.general.include_paths or "").split(PATH_DELIM)
            self.dirs_from_project_config = [Path.join(conf_base, p) for p in self.dirs_from_project_config if p]

        self.dirs_from_env = [Path.abspath(p) for p in os.environ.get("ENHAC_PATH", "").split(PATH_DELIM) if p]

        self._include_paths_lists = (
            [self.source_dir],
            self.dirs_from_directives,
            self.dirs_from_project_config,
            self.dirs_from_env,
        )

    @property
    def include_paths(self):
        return reduce(operator.add, self._include_paths_lists)

    def f_inc(self, path):
        path = sanitize(path)
        resolved = self.resolve(path, ["h"])

        path_h = resolved + ".h"
        path_c = resolved + ".c"
        if not Path.isfile(path_c):
            err = "Can't find corresponding .c file (%s)" % path_c
            self.enhac.p_warning(err)
            self.enhac.w_warning(err)
            path_c = None

        self.enhac.w_line(r'#include "%s"' % path_h)
        self.includes.append((path_h, path_c))

    def f_add_inc_path(self, path):
        path = sanitize(path)
        res_path = Path.join(self.source_dir, path)
        if not Path.isdir(res_path):
            self.enhac.p_warning("Include path %s -> %s is not valid directory." % (path, res_path))
        self.dirs_from_directives.append(res_path)

    def resolve(self, path, exts):
        for inc_path in self.include_paths:
            base_path = Path.join(inc_path, path)
            for ext in exts:
                res_path = base_path + "." + ext
                if not Path.isfile(res_path):
                    continue
                self.enhac.p_debug("Resolved %s to %s" % (path, base_path))
                return base_path
        path_lst_str = ":".join(self.include_paths)
        raise IOError("Can't find file to include %s in paths %s" % (path, path_lst_str))


def sanitize(path):
    return path.replace("'", "").replace('"', "").replace("\x00", "").replace("\n", "")


def nonzero(lst):
    return [x for x in lst if x]
